@@ -10,12 +10,14 @@ import (
 	"time"
 
 	"github.com/argoproj/gitops-engine/pkg/cache"
+	"github.com/argoproj/gitops-engine/pkg/sync/common"
 	"github.com/argoproj/gitops-engine/pkg/utils/kube"
 	"github.com/r3labs/diff"
 	log "github.com/sirupsen/logrus"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 	apierr "k8s.io/apimachinery/pkg/api/errors"
+	apimachineryvalidation "k8s.io/apimachinery/pkg/api/validation"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
@@ -33,6 +35,50 @@ const (
 	errDestinationMissing = "Destination server missing from app spec"
 )
 
+// AugmentSyncMsg enrich the K8s message with user-relevant information
+func AugmentSyncMsg(res common.ResourceSyncResult, apiResourceInfoGetter func() ([]kube.APIResourceInfo, error)) (string, error) {
+	switch res.Message {
+	case "the server could not find the requested resource":
+		resource, err := getAPIResourceInfo(res.ResourceKey.Group, res.ResourceKey.Kind, apiResourceInfoGetter)
+		if err != nil {
+			return "", fmt.Errorf("failed to get API resource info for group %q and kind %q: %w", res.ResourceKey.Group, res.ResourceKey.Kind, err)
+		}
+		if resource == nil {
+			res.Message = fmt.Sprintf("The Kubernetes API could not find %s/%s for requested resource %s/%s. Make sure the %q CRD is installed on the destination cluster.", res.ResourceKey.Group, res.ResourceKey.Kind, res.ResourceKey.Namespace, res.ResourceKey.Name, res.ResourceKey.Kind)
+		} else {
+			res.Message = fmt.Sprintf("The Kubernetes API could not find version %q of %s/%s for requested resource %s/%s. Version %q of %s/%s is installed on the destination cluster.", res.Version, res.ResourceKey.Group, res.ResourceKey.Kind, res.ResourceKey.Namespace, res.ResourceKey.Name, resource.GroupVersionResource.Version, resource.GroupKind.Group, resource.GroupKind.Kind)
+		}
+	}
+
+	return res.Message, nil
+}
+
+// getAPIResourceInfo gets Kubernetes API resource info for the given group and kind. If there's a matching resource
+// group _and_ kind, it will return the resource info. If there's a matching kind but no matching group, it will
+// return the first resource info that matches the kind. If there's no matching kind, it will return nil.
+func getAPIResourceInfo(group, kind string, getApiResourceInfo func() ([]kube.APIResourceInfo, error)) (*kube.APIResourceInfo, error) {
+
+	apiResources, err := getApiResourceInfo()
+
+	if err != nil {
+		return nil, fmt.Errorf("failed to get API resource info: %w", err)
+	}
+
+	for _, r := range apiResources {
+		if r.GroupKind.Group == group && r.GroupKind.Kind == kind {
+			return &r, nil
+		}
+	}
+
+	for _, r := range apiResources {
+		if r.GroupKind.Kind == kind {
+			return &r, nil
+		}
+	}
+
+	return nil, nil
+}
+
 // FormatAppConditions returns string representation of give app condition list
 func FormatAppConditions(conditions []argoappv1.ApplicationCondition) string {
 	formattedConditions := make([]string, 0)
@@ -62,6 +108,26 @@ func FilterByProjects(apps []argoappv1.Application, projects []string) []argoapp
 
 }
 
+// FilterByProjectsP returns application pointers which belongs to the specified project
+func FilterByProjectsP(apps []*argoappv1.Application, projects []string) []*argoappv1.Application {
+	if len(projects) == 0 {
+		return apps
+	}
+	projectsMap := make(map[string]bool)
+	for i := range projects {
+		projectsMap[projects[i]] = true
+	}
+	items := make([]*argoappv1.Application, 0)
+	for i := 0; i < len(apps); i++ {
+		a := apps[i]
+		if _, ok := projectsMap[a.Spec.GetProject()]; ok {
+			items = append(items, a)
+		}
+	}
+	return items
+
+}
+
 // FilterAppSetsByProjects returns applications which belongs to the specified project
 func FilterAppSetsByProjects(appsets []argoappv1.ApplicationSet, projects []string) []argoappv1.ApplicationSet {
 	if len(projects) == 0 {
@@ -95,6 +161,20 @@ func FilterByRepo(apps []argoappv1.Application, repo string) []argoappv1.Applica
 	return items
 }
 
+// FilterByRepoP returns application pointers
+func FilterByRepoP(apps []*argoappv1.Application, repo string) []*argoappv1.Application {
+	if repo == "" {
+		return apps
+	}
+	items := make([]*argoappv1.Application, 0)
+	for i := 0; i < len(apps); i++ {
+		if apps[i].Spec.GetSource().RepoURL == repo {
+			items = append(items, apps[i])
+		}
+	}
+	return items
+}
+
 // FilterByCluster returns an application
 func FilterByCluster(apps []argoappv1.Application, cluster string) []argoappv1.Application {
 	if cluster == "" {
@@ -124,6 +204,22 @@ func FilterByName(apps []argoappv1.Application, name string) ([]argoappv1.Applic
 	return items, status.Errorf(codes.NotFound, "application '%s' not found", name)
 }
 
+// FilterByNameP returns pointer applications
+// This function is for the changes in #12985.
+func FilterByNameP(apps []*argoappv1.Application, name string) []*argoappv1.Application {
+	if name == "" {
+		return apps
+	}
+	items := make([]*argoappv1.Application, 0)
+	for i := 0; i < len(apps); i++ {
+		if apps[i].Name == name {
+			items = append(items, apps[i])
+			return items
+		}
+	}
+	return items
+}
+
 // RefreshApp updates the refresh annotation of an application to coerce the controller to process it
 func RefreshApp(appIf v1alpha1.ApplicationInterface, name string, refreshType argoappv1.RefreshType) (*argoappv1.Application, error) {
 	metadata := map[string]interface{}{
@@ -732,6 +828,21 @@ func ContainsSyncResource(name string, namespace string, gvk schema.GroupVersion
 	return false
 }
 
+// IncludeResource checks if an app resource matches atleast one of the filters, then it returns true.
+func IncludeResource(resourceName string, resourceNamespace string, gvk schema.GroupVersionKind,
+	syncOperationResources []*argoappv1.SyncOperationResource) bool {
+	for _, syncOperationResource := range syncOperationResources {
+		includeResource := syncOperationResource.Compare(resourceName, resourceNamespace, gvk)
+		if syncOperationResource.Exclude {
+			includeResource = !includeResource
+		}
+		if includeResource {
+			return true
+		}
+	}
+	return false
+}
+
 // NormalizeApplicationSpec will normalize an application spec to a preferred state. This is used
 // for migrating application objects which are using deprecated legacy fields into the new fields,
 // and defaulting fields in the spec (e.g. spec.project)
@@ -954,3 +1065,36 @@ func AppInstanceNameFromQualified(name string, defaultNs string) string {
 func ErrProjectNotPermitted(appName, appNamespace, projName string) error {
 	return fmt.Errorf("application '%s' in namespace '%s' is not permitted to use project '%s'", appName, appNamespace, projName)
 }
+
+// IsValidPodName checks that a podName is valid
+func IsValidPodName(name string) bool {
+	// https://github.com/kubernetes/kubernetes/blob/976a940f4a4e84fe814583848f97b9aafcdb083f/pkg/apis/core/validation/validation.go#L241
+	validationErrors := apimachineryvalidation.NameIsDNSSubdomain(name, false)
+	return len(validationErrors) == 0
+}
+
+// IsValidAppName checks if the name can be used as application name
+func IsValidAppName(name string) bool {
+	// app names have the same rules as pods.
+	return IsValidPodName(name)
+}
+
+// IsValidProjectName checks if the name can be used as project name
+func IsValidProjectName(name string) bool {
+	// project names have the same rules as pods.
+	return IsValidPodName(name)
+}
+
+// IsValidNamespaceName checks that a namespace name is valid
+func IsValidNamespaceName(name string) bool {
+	// https://github.com/kubernetes/kubernetes/blob/976a940f4a4e84fe814583848f97b9aafcdb083f/pkg/apis/core/validation/validation.go#L262
+	validationErrors := apimachineryvalidation.ValidateNamespaceName(name, false)
+	return len(validationErrors) == 0
+}
+
+// IsValidContainerName checks that a containerName is valid
+func IsValidContainerName(name string) bool {
+	// https://github.com/kubernetes/kubernetes/blob/53a9d106c4aabcd550cc32ae4e8004f32fb0ae7b/pkg/api/validation/validation.go#L280
+	validationErrors := apimachineryvalidation.NameIsDNSLabel(name, false)
+	return len(validationErrors) == 0
+}
